require "spec_helper"
require "toni/builder"

describe Toni::Builder do

  let(:builder) { Toni::Builder.new("") }

  describe :role do
    it "creates a new role instance" do
      expect(builder.role(:user)).to be_instance_of(Toni::Role)
    end
  end

  describe :roles do
    it "return hash of defined roles" do
      builder.role(:user)
      expect(builder.roles.keys).to eq([:user])
    end
  end

end

describe Toni::Builder::RoleBuilder do

  let(:builder) { Toni::Builder::RoleBuilder.new(:user) }

  describe :role do
    it "is a Role" do
      expect(builder.role).to be_instance_of(Toni::Role)
    end
  end

  describe :includes do
    it "calls add_ancestor for the given role_symbol" do
      expect(builder.role).to receive(:add_ancestor).with(:parent_user)
      builder.includes(:parent_user)
    end
  end

  describe :has_permission_on do
    it "calls add_permission on role" do
      expect(builder.role).to receive(:add_permission)
      builder.has_permission_on(:my_resource)
    end
  end

end

describe Toni::Builder::PermissionBuilder do

  let(:builder) { Toni::Builder::PermissionBuilder.new(:resource_name) }

  describe :initialize do
    it "sets the activtities" do
      builder = Toni::Builder::PermissionBuilder.new(:resource_name, to: [:create, :read])
      expect(builder.permission.activities).to eq([:create, :read])
    end
    it "sets single activtities" do
      builder = Toni::Builder::PermissionBuilder.new(:resource_name, to: :read)
      expect(builder.permission.activities).to eq([:read])
    end
  end

  describe :permission do
    it "is a Permission" do
      expect(builder.permission).to be_instance_of(Toni::Permission)
    end
  end

  describe :expect_attribute do
    it "calls expect_method with is_attribute true" do
      expect(builder).to receive(:expect_method).with(nil, true)
      builder.expect_attribute(nil)
    end
  end

  describe :expect_method do
    it "calls add_matcher on permission per method_name" do
      expect(builder.permission).to receive(:add_matcher).with(:foo, anything, false)
      expect(builder.permission).to receive(:add_matcher).with(:bar, anything, false)
      builder.expect_method(:foo) { nil }
      builder.expect_method(:bar) { nil }
    end
  end

end

describe Toni::Builder::ExpectationBuilder do

  let(:builder) { Toni::Builder::ExpectationBuilder.new }

  describe :execute do
    it "calls block lazely" do
      double = double()
      builder = Toni::Builder::ExpectationBuilder.new do
        double.call
      end
      expect(double).to receive(:call).once
      builder.execute
    end
  end

  describe "uses rspec matchers" do
    it "calls rspec matcher methods" do
      allow(RSpec::Matchers).to receive(:be_nil)
      builder.be_nil
    end

    it "sets the called matcher" do
      builder = Toni::Builder::ExpectationBuilder.new do
        be(:cool)
      end
      expect(builder.execute).to be_instance_of(RSpec::Matchers::BuiltIn::Equal)
    end
  end

  describe :not do
    it "should create a NotMatcher" do
      expect(builder.not).to be_instance_of(Toni::PermissionMatcher::NotMatcher)
    end
  end

  describe :permitted_to do
    it "should create a PermittedToMatcher" do
      expect(builder.permitted_to(:read)).to be_instance_of(Toni::PermissionMatcher::PermittedToMatcher)
    end
  end

  describe :current_user do
    it "gets the toni user" do
      expect(Toni).to receive(:current_user)
      builder.current_user
    end
  end

end