require "toni"
require "toni/in_model"
require "active_record"
require "spec_helper"

describe Toni::InModel do
  let(:record_class) do
    Class.new(ActiveRecord::Base) do
      def self.table_name
        :test
      end
      def self.authorization_context
        :resource_name
      end
    end
  end

  let(:record) { record_class.new }

  before do
    record_class.destroy_all
    allow(Toni).to receive(:current_roles) { [] }
  end

  context :before_filter do
    it "adds before filter on include" do
      expect(record_class).to receive(:before_destroy)
      expect(record_class).to receive(:before_create)
      expect(record_class).to receive(:before_update)
      expect(record_class).to receive(:after_find)
      record_class.send(:include, Toni::InModel)
    end

    it "calls permitted_to? if records gets found" do
      record_class.send(:include, Toni::InModel)
      Toni.without_authorization { record.save }
      allow(Toni).to receive(:permitted_to?) { true }
      expect(Toni).to receive(:permitted_to?).with(:read, record_class, bang: true)
      record_class.find(record.id)
    end

    it "calls permitted_to? if records gets created" do
      record_class.send(:include, Toni::InModel)
      allow(Toni).to receive(:permitted_to?) { true }
      expect(Toni).to receive(:permitted_to?).with(:create, record_class, bang: true)
      record.save
    end

    it "calls permitted_to? if records gets updated" do
      record_class.send(:include, Toni::InModel)
      allow(Toni).to receive(:permitted_to?) { true }
      record.save
      record.foo = :x
      expect(Toni).to receive(:permitted_to?).with(:update, record_class, bang: true)
      record.save
    end

    it "calls permitted_to? if records gets destroyes" do
      record_class.send(:include, Toni::InModel)
      allow(Toni).to receive(:permitted_to?) { true }
      record.save
      expect(Toni).to receive(:permitted_to?).with(:delete, record_class, bang: true)
      record.destroy
    end

    it "does not create record if permitted_to bangs" do
      record_class.send(:include, Toni::InModel)
      allow(Toni).to receive(:permitted_to?) { raise Toni::NotAuthorizedError }
      expect { record.save }.to raise_error(Toni::NotAuthorizedError)
      expect(record_class.count).to eq(0)
    end

    it "does not update record if permitted_to bangs" do
      record_class.send(:include, Toni::InModel)
      Toni.without_authorization { record.save }
      allow(Toni).to receive(:permitted_to?) { raise Toni::NotAuthorizedError }
      record.foo = :x
      expect { record.save }.to raise_error(Toni::NotAuthorizedError)
      expect(record_class.count).to eq(1)
      allow(Toni).to receive(:permitted_to?) { true }
      expect(record_class.first.foo).to be_nil
    end

    it "does not destroy record if permitted_to bangs" do
      record_class.send(:include, Toni::InModel)
      Toni.without_authorization { record.save }
      allow(Toni).to receive(:permitted_to?) { raise Toni::NotAuthorizedError }
      expect { record.destroy }.to raise_error(Toni::NotAuthorizedError)
      expect(record_class.count).to eq(1)
    end
  end

  context :scope do

    let(:matcher) { RSpec::Matchers::BuiltIn::BeNil.new }
    let(:matcher2) { RSpec::Matchers::BuiltIn::Eq.new("x") }
    let(:matcher3) { RSpec::Matchers::BuiltIn::Eq.new("y") }
    let(:permission) do
      p = Toni::Permission.new(:resource_name, [:read, :create])
      p.add_matcher(:foo, matcher, true)
      p.add_matcher(:bar, matcher2, true)
      p
    end
    let(:permission2) do
      p = Toni::Permission.new(:resource_name, [:read])
      p.add_matcher(:foo, matcher3, true)
      p
    end

    it "creates a scope" do
      expect(record_class).to receive(:scope).with(:permitted_to, anything)
      record_class.send(:include, Toni::InModel)
    end

    it "creates a useful scope" do
      record_class.send(:include, Toni::InModel)
      allow(Toni).to receive(:current_permissions_for).with(:read, :resource_name) do
        [permission, permission2]
      end
      expect(record_class.send(:scope_for_filter_permissions)).to eq(["((foo IS NULL) AND (bar = ?)) OR ((foo = ?))", "x", "y"])
    end

    it "scopes permitted_to" do
      allow(Toni).to receive(:current_permissions_for).with(:read, :resource_name) do
        [permission, permission2]
      end
      record_class.send(:include, Toni::InModel)
      allow(Toni).to receive(:permitted_to?) { true }
      r1 = record_class.create(foo: "x", bar: "y")
      r2 = record_class.create(foo: nil, bar: "x")
      r3 = record_class.create(foo: "y", bar: "something")
      expect(record_class.permitted_to.map(&:id)).to eq([r2.id, r3.id])
    end
  end
end