require "spec_helper"
require "toni/permission_matcher"
require "toni"
require "ostruct"

describe Toni::PermissionMatcher do

  let(:matcher) do
    Class.new do
      def matches?(actual)
        false
      end
    end.new
  end
  let(:permission_matcher) { Toni::PermissionMatcher::Base.new(:foo, matcher) }

  describe :method_name do
    it "sets the method_name on initialize" do
      expect(Toni::PermissionMatcher::Base.new(:method_name, nil).method_name).to eq(:method_name)
    end
  end

  describe :matcher do
    it "sets the matcher on initialize" do
      expect(permission_matcher.matcher).to eq(matcher)
    end
  end

  describe :is_attr do
    it "sets the is_attr on initialize" do
      expect(permission_matcher.is_attr).to eq(false)
    end
  end

  describe :filter_condition do
    let(:expectation_builder) { Toni::Builder::ExpectationBuilder.new }
    let(:matcher2) { expectation_builder.eq("test") }
    let(:permission_matcher) { Toni::PermissionMatcher::Base.new(:foo, matcher2, true) }

    it "return nil if is_attr is false" do
      permission_matcher = Toni::PermissionMatcher::Base.new(:foo, matcher2, false)
      expect(permission_matcher.filter_condition).to eq(nil)
    end

    it "gets filter condition from filterable permission matcher if it exist" do
      expect_any_instance_of(Toni::PermissionFilter::Eq).to receive(:filter_condition)
      permission_matcher.filter_condition
    end

    it "raises error when permission matcher is not available" do
      allow(permission_matcher).to receive(:permission_filter) { nil }
      expect { permission_matcher.filter_condition }.to raise_error(Toni::MissingPermissionFilterError)
    end

    it "uses filter_condition method of matcher if matcher responds" do
      allow(matcher2).to receive(:respond_to?) { true }
      expect(matcher2).to receive(:filter_condition) { "condition" }
      expect(permission_matcher.filter_condition).to eq("condition")
    end
  end

  describe :matches? do
    let(:object) do
      Class.new do
        def self.authorization_context
          :resource_name
        end
        def foo
          nil
        end
      end.new
    end
    let(:object2) do
      Class.new do
        def self.authorization_context
          :resource_name
        end
      end.new
    end

    it "raises NoMethodForMatcherError if method is not there" do
      expect { permission_matcher.matches?(object2) }.to raise_error(Toni::NoMethodForMatcherError)
    end

    it "returns false if matcher does not match" do
      expect(permission_matcher.matches?(object)).to eq(false)
    end

    it "returns true if matcher does match" do
      allow(matcher).to receive(:matches?) { true }
      expect(permission_matcher.matches?(object)).to eq(true)
    end

    it "returns true if matcher is rspec nil matcher and value is nil" do
      matcher = RSpec::Matchers::BuiltIn::BeNil.new
      permission_matcher = Toni::PermissionMatcher::Base.new(:foo, matcher)
      expect(permission_matcher.matches?(object)).to eq(true)
    end

    it "returns false if matcher is rspec nil matcher and value is not nil" do
      allow(object).to receive(:foo) { "a string" }
      matcher = RSpec::Matchers::BuiltIn::BeNil.new
      permission_matcher = Toni::PermissionMatcher::Base.new(:foo, matcher)
      expect(permission_matcher.matches?(object)).to eq(false)
    end
  end

end

describe Toni::PermissionMatcher::NotMatcher do

  let(:not_matcher) { Toni::PermissionMatcher::NotMatcher.new }

  describe :method_missing do
    it "looks up method in ExpectationBuilder" do
      expect { not_matcher.be_truthy }.to_not raise_error
    end
    it "throws error when method does not exist" do
      expect { not_matcher.that_does_not_exist }.to raise_error
    end
  end

  describe :matches? do
    it "raises error when matcher is nil" do
      expect { not_matcher.matches?(nil) }.to raise_error(Toni::MissingMatcherError)
    end
    it "uses matches method of matcher and flips the boolean" do
      not_matcher.eq(nil)
      expect(not_matcher.matches?(nil)).to eq(false)
    end
  end

  describe :filter_condition do
    it "gets filter condition from child matcher if it responds to filter_condition_inverse" do
      not_matcher.be_truthy
      expect_any_instance_of(Toni::PermissionFilter::BeTruthy).to receive(:filter_condition_inverse)
      not_matcher.filter_condition
    end

    it "raises error when child matcher does not respond to filter_condition_inverse" do
      not_matcher.be_truthy
      allow_any_instance_of(Toni::PermissionFilter::BeTruthy).to receive(:respond_to?) { false }
      expect { not_matcher.filter_condition }.to raise_error(Toni::MissingPermissionFilterError)
    end
  end

end

describe Toni::PermissionMatcher::PermittedToMatcher do

  let(:permitted_to_matcher) { Toni::PermissionMatcher::PermittedToMatcher.new(:read) }

  describe :matches? do
    it "calls Toni.permitted_to? for given attribute" do
      double = double
      expect(Toni).to receive(:permitted_to?).with(:read, double)
      permitted_to_matcher.matches?(double)
    end
  end

end