require "spec_helper"
require "toni/permission"
require "ostruct"

describe Toni::Permission do

  let(:permission) { Toni::Permission.new(:my_resource) }

  describe :resource_name do
    it "sets the resource_name on initialize" do
      expect(Toni::Permission.new(:a_resource).resource_name).to eq(:a_resource)
    end
  end

  describe :activities do
    it "sets the activities on initialize" do
      expect(Toni::Permission.new(:a_resource, [:read]).activities).to eq([:read])
    end
  end

  describe :add_matcher do
    it "adds the matcher" do
      permission.add_matcher(:foo, nil)
      expect(permission.matchers.first).to be_instance_of(Toni::PermissionMatcher::Base)
    end
  end

  describe :permission_for? do
    let(:permission) do
      Toni::Permission.new(:resource_name, [:read, :create])
    end
    it "should be allowed to read resource_name" do
      expect(permission.permission_for?(:read, :resource_name)).to eq(true)
    end
    it "should be allowed to delete resource_name" do
      expect(permission.permission_for?(:delete, :resource_name)).to eq(false)
    end
    it "should be allowed to read xxx" do
      expect(permission.permission_for?(:read, :xxx)).to eq(false)
    end
  end

  describe :permitted_to? do

    let(:permission) do
      Toni::Permission.new(:resource_name, [:read, :create])
    end

    it "calls permission_for?" do
      expect(permission).to receive(:permission_for?).with(:read, :resource_name) { true }
      expect(permission.permitted_to?(:read, :resource_name)).to eq(true)
    end

    context "with matchers" do
      let(:matcher) { RSpec::Matchers::BuiltIn::BeNil.new }
      let(:permission) do
        p = Toni::Permission.new(:resource_name, [:read, :create])
        p.add_matcher(:foo, matcher)
        p
      end
      let(:object) do
        Class.new do
          def self.authorization_context
            :resource_name
          end
        end.new
      end

      it "always is true when only resource name is given" do
        allow(permission.matchers.first).to receive(:execute) do
          Class.new do
            def matches?(object)
              false
            end
          end.new
        end
        expect(permission.permitted_to?(:read, :resource_name)).to eq(true)
      end

      it "is false when object is given" do
        allow(permission.matchers.first).to receive(:execute) do
          Class.new do
            def matches?(object)
              false
            end
          end.new
        end
        expect(permission.permitted_to?(:read, object)).to eq(false)
      end
    end

    context "with resource object" do
      let(:object) do
        Class.new do
          def self.authorization_context
            :resource_name
          end
        end.new
      end
      let(:object2) do
        Class.new do
          def self.authorization_context
            :xxx
          end
        end.new
      end
      let(:object3) do
        Class.new.new
      end

      it "should be allowed to read object" do
        expect(permission.permitted_to?(:read, object)).to eq(true)
      end
      it "should be allowed to delete object" do
        expect(permission.permitted_to?(:delete, object)).to eq(false)
      end
      it "should be allowed to read xxx" do
        expect(permission.permitted_to?(:read, object2)).to eq(false)
      end
    end
  end

end