require "spec_helper"
require "toni/role"
require "toni/permission"
require "ostruct"

describe Toni::Role do

  let(:role) { Toni::Role.new(:user)}
  let(:permission) { Toni::Permission.new(:resource_name)}

  describe :role_symbol do
    it "sets the role_symbol on initialize" do
      expect(Toni::Role.new(:everything).role_symbol).to eq(:everything)
    end
  end

  describe :add_ancestor do
    it "adds role symbol to ancestors" do
      role.add_ancestor(:another)
      expect(role.ancestors).to eq([:another])
    end

    it "adds role symbol only once" do
      role.add_ancestor(:another)
      role.add_ancestor(:another)
      expect(role.ancestors).to eq([:another])
    end
  end

  describe :add_permission do
    it "adds permission to permissions" do
      role.add_permission(permission)
      expect(role.permissions).to eq([permission])
    end
  end

  describe :permissions_for do

    let(:permission) { Toni::Permission.new(:resource_name, [:read, :create]) }
    let(:permission2) { Toni::Permission.new(:other_resource, [:read, :create]) }

    let(:role) do
      role = Toni::Role.new(:user)
      role.add_permission(permission)
      role.add_permission(permission2)
      role
    end

    it "returns all permissions that are belonging to resource and activity" do
      allow(permission).to receive(:permission_for?).with(:read, :resource_name) { true }
      allow(permission2).to receive(:permission_for?).with(:read, :resource_name) { false }
      expect(role.permissions_for(:read, :resource_name)).to eq([permission])
    end

  end

  describe :permitted_to? do

    let(:permission) do
      Toni::Permission.new(:resource_name, [:read, :create])
    end

    let(:role) do
      role = Toni::Role.new(:user)
      role.add_permission(permission)
      role
    end

    let(:object) do
      Class.new.new
    end

    it "should raise NoAuthorizationContextProvidedError" do
      expect { role.permitted_to?(:read, object) }.to raise_error(Toni::NoAuthorizationContextProvidedError)
    end

    it "should call permission permitted_to? and return true if permission is true" do
      allow(permission).to receive(:permitted_to?).with(:read, :resource_name, anything) { true }
      expect(role.permitted_to?(:read, :resource_name)).to eq(true)
    end

    it "should call permission permitted_to? and return false if permission is false" do
      allow(permission).to receive(:permitted_to?).with(:read, :resource_name, anything) { false }
      expect(role.permitted_to?(:read, :resource_name)).to eq(false)
    end

    context "inheritance" do
      let(:parent_role) do
        role = Toni::Role.new(:parent_user)
        role.add_permission(permission)
        role
      end

      let(:role) do
        role = Toni::Role.new(:user)
        role.add_ancestor(parent_role)
        role
      end

      it "uses permission from parent role" do
        allow(permission).to receive(:permitted_to?).with(:read, :resource_name, anything) { true }
        expect(role.permitted_to?(:read, :resource_name)).to eq(true)
      end
    end
  end

end