require "spec_helper"
require "toni"
require "ostruct"

describe Toni do

  let(:role_symbols) { [] }
  let(:current_user) { OpenStruct.new(role_symbols: role_symbols) }

  before do
    allow(File).to receive(:read).with(Toni::AUTH_FILE) do
      <<-eos
        role :guest do
          has_permission_on :x, to: [:read, :delete]
        end
        role :user do
          has_permission_on :test, to: [:read, :create]
        end
      eos
    end
  end

  describe ".current_user" do
    it "has a anonymous user as default" do
      expect(Toni.current_user).to be_instance_of(Toni::AnonymousUser)
    end
  end

  describe ".current_user=" do
    it "raises an error when current user does not respond to role_symbols" do
      expect{ Toni.current_user = OpenStruct.new }.to raise_error(Toni::InvalidCurrentUserError)
    end

    it "sets current user respond to role_symbols" do
      Toni.current_user = current_user
      expect(Toni.current_user).to eq(current_user)
    end

    it "uses an anonymous user when current user was set to nil" do
      Toni.current_user = nil
      expect(Toni.current_user).to be_instance_of(Toni::AnonymousUser)
    end
  end

  describe ".roles" do
    it "reads roles from auth_file" do
      expect(Toni.roles.keys).to eq([:guest, :user])
    end
  end

  describe ".current_roles" do
    let(:role_symbols) { [:user] }
    it "gets the roles that are in the role_symbols methods" do
      Toni.current_user = current_user
      expect(Toni.current_roles).to eq([Toni.roles[:user]])
    end
  end

  describe ".current_permissions_for" do
    let(:user_role) { Toni::Role.new(:user) }
    let(:seller_role) { Toni::Role.new(:seller) }
    let(:permission1) { Toni::Permission.new(:books, [:read]) }
    let(:permission2) { Toni::Permission.new(:books, [:read]) }
    it "gets the permissions that are assigned to current roles for a specific resource" do
      allow(Toni).to receive(:current_roles) { [user_role, seller_role] }
      expect(user_role).to receive(:permissions_for).with(:read, :books) { [permission1] }
      expect(seller_role).to receive(:permissions_for).with(:read, :books) { [permission2] }
      expect(Toni.current_permissions_for(:read, :books)).to eq([permission1, permission2])
    end
  end

  describe ".without_authorization" do
    it "executes block without authorization and returns result of the given block" do
      with_authorization = Toni.permitted_to?(:something, :test)
      without_authorization = false
      expect(Toni.without_authorization do
        without_authorization = Toni.permitted_to?(:something, :test)
        3
      end).to eq(3)
      expect(with_authorization).to eq(false)
      expect(without_authorization).to eq(true)
    end
  end

  describe ".permitted_to?" do
    let(:role_symbols) { [:user, :seller] }
    let(:user_role) { Toni::Role.new(:user) }
    let(:seller_role) { Toni::Role.new(:seller) }
    let(:roles) { { user: user_role, seller: seller_role } }

    before do
      allow(Toni).to receive(:roles) { roles }
      Toni.current_user = current_user
    end

    it "is not permitted_to if all roles return false" do
      allow(user_role).to   receive(:permitted_to?).with(:read, :resource, anything) { false }
      allow(seller_role).to receive(:permitted_to?).with(:read, :resource, anything) { false }
      expect(Toni.permitted_to?(:read, :resource)).to eq(false)
    end

    it "is permitted_to if at least user_role returns true" do
      allow(user_role).to   receive(:permitted_to?).with(:read, :resource, anything) { true }
      allow(seller_role).to receive(:permitted_to?).with(:read, :resource, anything) { false }
      expect(Toni.permitted_to?(:read, :resource)).to eq(true)
    end

    it "raises error if it is not permitted to and bang is true" do
      allow(user_role).to   receive(:permitted_to?).with(:read, :resource, anything) { false }
      allow(seller_role).to receive(:permitted_to?).with(:read, :resource, anything) { false }
      expect { Toni.permitted_to?(:read, :resource, bang: true) }.to raise_error(Toni::NotAuthorizedError)

    end

  end

end